/* -*-mode: C; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  dialogs.c - Code for the dialog boxes we use in the settings.
 *
 *  GTKCamShot - A simple webcam grabbing application
 *
 * Homepage:
 *   http://www.steve.org.uk/Software/gtkcam
 *
 * Author:
 *  Steve Kemp <steve@steve.org.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */



#include "gtkcamshot.h"


GtkWidget *tmp_entry;
GtkWidget *dir_entry;
GtkWidget *format_entry;

/*
 * Combo box for holding the chosen font
 */
GtkWidget *font_entry;
GtkWidget *font_format;


/**
 * Called when the user presses OK on the preferences page.
 */
static void on_ok( GtkButton   *button,
		   GtkWindow *window )
{

    /* Get the text. */
    const gchar *u = gtk_entry_get_text( GTK_ENTRY(dir_entry) );
    const gchar *d = gtk_entry_get_text( GTK_ENTRY(tmp_entry) );
    const gchar *t = gtk_entry_get_text( GTK_ENTRY(format_entry) );


    /* Setup output directory. */
    memset( g_output_dir, sizeof(g_output_dir), '\0' );
    strncpy( g_output_dir, u, sizeof(g_output_dir)-2 );

    if ( u[ strlen(u) ] != '/' )
	strcat( g_output_dir, "/");
    
    /* Setup temporary file */
    memset( g_tmp_file, sizeof(g_tmp_file), '\0' );
    strncpy( g_tmp_file, d, sizeof(g_tmp_file)-1 );


    /* Setup format string */
    memset( g_format_string, sizeof( g_format_string), '\0' );
    strncpy( g_format_string, t, sizeof(g_format_string)-1 );

    /*
     * TODO - Allow user to change size from 16 pixels
     * Now handle the chosen font. 
     */
    snprintf(g_timestamp_font,sizeof(g_timestamp_font), "%s/16",
	     (char*)gtk_entry_get_text( (GTK_COMBO (font_entry)->entry )));


    gtk_widget_destroy(gtk_widget_get_parent(GTK_WIDGET(gtk_widget_get_parent(GTK_WIDGET(window)))));

  
}



/**
 * Called when the user presses cancel on the preferences page.
 */
static void on_cancel( GtkButton   *button,
		       GtkWindow   *window )
{
    gtk_widget_destroy(gtk_widget_get_parent(GTK_WIDGET(gtk_widget_get_parent(GTK_WIDGET(window)))));
}



void show_preferences_window()
{
    GtkWidget *window;
    GtkWidget *button;
    GtkWidget *table;
    GtkWidget *notebook;
    GtkWidget *label;
    int i;
   
    
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "GtkCamShot - Preferences" );


    g_signal_connect (G_OBJECT (window), "delete_event",
	              G_CALLBACK (on_cancel), NULL);
    
    gtk_container_set_border_width (GTK_CONTAINER (window), 10);

    table = gtk_table_new (3, 6, FALSE);
    gtk_container_add (GTK_CONTAINER (window), table);
    
    /* Create a new notebook, place the position of the tabs */
    notebook = gtk_notebook_new ();
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
    gtk_table_attach_defaults (GTK_TABLE (table), notebook, 0, 6, 0, 1);
    gtk_widget_show (notebook);
    

    /**
     * Path page.
     */
    {
      GtkWidget *dialog, *tmp_label;
      GtkWidget *urlbox, *dirbox,  *dir_label, *vbox;

      dialog = gtk_frame_new("Temporary Files and Paths");

      dir_label = gtk_label_new ("Output Directory:");
      tmp_label = gtk_label_new ("Temporary File:");

      dir_entry = gtk_entry_new ();
      gtk_entry_set_max_length ( GTK_ENTRY(dir_entry), 100);
      gtk_entry_set_text( GTK_ENTRY(dir_entry), g_output_dir );

      tmp_entry = gtk_entry_new ();
      gtk_entry_set_max_length ( GTK_ENTRY(tmp_entry), 100);
      gtk_entry_set_text( GTK_ENTRY(tmp_entry), g_tmp_file );

      dirbox = gtk_hbox_new (FALSE, 0);
      urlbox = gtk_hbox_new (FALSE, 0);

      vbox   = gtk_vbox_new( FALSE, 0);

      gtk_box_pack_start (GTK_BOX (urlbox), dir_label, TRUE, TRUE, 0);
      gtk_box_pack_end (GTK_BOX (urlbox), dir_entry, TRUE, TRUE, 0);


      gtk_box_pack_start (GTK_BOX (dirbox), tmp_label, TRUE, TRUE, 0);
      gtk_box_pack_end (GTK_BOX (dirbox), tmp_entry, TRUE, TRUE, 0);


      gtk_box_pack_start (GTK_BOX (vbox), dirbox, TRUE, TRUE, 0);
      gtk_box_pack_end (GTK_BOX (vbox), urlbox, TRUE, TRUE, 0);
      
      gtk_container_add (GTK_CONTAINER (GTK_FRAME(dialog)),
			  vbox);

      gtk_widget_show_all (dialog);

      label = gtk_label_new ("Paths");
      gtk_notebook_append_page (GTK_NOTEBOOK (notebook), dialog, label);

    }

    /**
     * Timestamp / Font page
     */
    {
      GtkWidget *dialog, *vbox;
      GtkWidget *font_label, *font_box;
      GtkWidget *format_label, *format_box;
      char **avail;
      int available_font_count = 0;

      GList *glist = NULL;

      imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/B/");

      avail = imlib_list_fonts(&available_font_count);

      printf("There are %d fonts available\n", available_font_count);
      for( i = 0; i < available_font_count; i++ )
      {
	glist = g_list_append (glist, avail[i] );
      }

      dialog = gtk_frame_new("Timestamp Setup");

      font_label = gtk_label_new ("Font:");
      font_entry = gtk_combo_new ();
      gtk_combo_set_popdown_strings (GTK_COMBO (font_entry), glist);
    
      font_box = gtk_hbox_new (FALSE, 0);

      format_label = gtk_label_new("Timestamp Format");
      format_entry = gtk_entry_new ();
      gtk_entry_set_max_length ( GTK_ENTRY(format_entry), 100);
      gtk_entry_set_text( GTK_ENTRY(format_entry), g_format_string );

      format_box = gtk_hbox_new (FALSE, 0);

      vbox   = gtk_vbox_new( FALSE, 0);

      gtk_box_pack_start(GTK_BOX (font_box), font_label, TRUE, TRUE, 0);
      gtk_box_pack_end(GTK_BOX (font_box), font_entry, TRUE, TRUE, 0);

      gtk_box_pack_start(GTK_BOX (format_box), format_label, TRUE, TRUE, 0);
      gtk_box_pack_end (GTK_BOX (format_box), format_entry, TRUE, TRUE, 0);


      gtk_box_pack_start (GTK_BOX (vbox), font_box, TRUE, TRUE, 0);
      gtk_box_pack_end (GTK_BOX (vbox), format_box, TRUE, TRUE, 0);
      
      gtk_container_add (GTK_CONTAINER (GTK_FRAME(dialog)),
			  vbox);

      gtk_widget_show_all (dialog);

      label = gtk_label_new ("Timestamps");
      gtk_notebook_append_page (GTK_NOTEBOOK (notebook), dialog, label);

    }



    /* Create a bunch of buttons */
    button = gtk_button_new_with_label ("cancel");
    g_signal_connect_swapped (G_OBJECT (button), "clicked",
			      G_CALLBACK (on_cancel), window);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 0, 1, 1, 2);
    gtk_widget_show (button);
    
    
    button = gtk_button_new_with_label ("OK");

    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       GTK_SIGNAL_FUNC (on_ok), window);

    gtk_table_attach_defaults (GTK_TABLE (table), button, 3, 4, 1, 2);
    gtk_widget_show (button);
   
    gtk_widget_show (table);
    gtk_widget_show (window);
    
}

