/* -*-mode: C; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  gtkcamshot.c - Main code.
 *
 *  GTKCamShot - A simple webcam grabbing application
 *
 * Homepage:
 *   http://www.steve.org.uk/Software/gtkcam
 *
 * Author:
 *  Steve Kemp <steve@steve.org.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>


#include "gtkcamshot.h"

#ifndef DEFAULT_FONT
# define DEFAULT_FONT "/var/lib/defoma/fontconfig.d/B/Bitstream-Vera-Sans-Bold.ttf"
#endif




/*
 * Count of images per session.
 */
static int count = 1;

/*
 * Temporary file name.
 */
char g_tmp_file[256];


/*
 * Output directory.
 */
char g_output_dir[256];


/*
 * Timestampe font.
 */
char g_timestamp_font[256];

/*
 * Timestamp format string.
 */
char g_format_string[256];

/*
 * View a timestamp on the images?
 */
int g_view_timestamp;


/*
 * Global variables.
 */
GtkWidget *window;        /* Main window */
GtkWidget *imageview;     /* Image displaying widget */






/**
 * Download the current webcam image, and update the image
 * object to display it.
 */
gint get_image(gpointer data)
{
    gdk_threads_enter();
    
    Imlib_Image image;
    Imlib_Font font;
    int width, height;


    /* Grab an image */
    image = grab_one(&width, &height);
	imlib_context_set_image(image);

    if ( NULL != image )
    {
	if ( g_view_timestamp )
	{
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/A/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/B/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/C/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/D/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/E/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/F/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/G/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/H/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/I/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/J/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/K/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/L/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/M/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/N/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/O/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/P/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/Q/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/R/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/S/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/T/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/U/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/V/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/W/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/X/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/Y/");
	    imlib_add_path_to_font_path("/var/lib/defoma/fontconfig.d/Z/");

	    font = imlib_load_font( g_timestamp_font );
	    if (font)
	    {
		int text_w, text_h;
		time_t now;
		struct tm *loctime;

		char text[4096];

		memset(text,'\0', sizeof(text));

		now = time(NULL);
		loctime = localtime( &now );

		strftime(text,sizeof(text), g_format_string, loctime );

		imlib_context_set_font(font);
		imlib_get_text_size(text, &text_w, &text_h); 

		/* Text colour */
		imlib_context_set_color(0, 0, 0, 255);

		/* Draw the text */
		imlib_text_draw(20, 240 - (text_h / 2), text );
		imlib_free_font();
	    }
	    else
	    {
		printf("Failed to load font: %s\n", g_timestamp_font);
	    }
	}
	save_image(image, g_tmp_file);
	gib_imlib_free_image_and_decache(image);
	gtk_image_set_from_file(GTK_IMAGE(imageview), g_tmp_file );
    }

    gdk_threads_leave();
    return( TRUE);
}



/**
 * Save the current image to the output directory.
 */
void save_snapshot() 
{
    char buf[1024];
    snprintf(buf, sizeof(buf), "/bin/cp %s %sout%d.jpg", 
	     g_tmp_file, g_output_dir, count );
    system(buf);
    count+=1;
}



/**
 * Handle mouse clicks on the applications main window.
 */
static gint button_press_event( GtkWidget      *widget,
                                GdkEventButton *event )
{
  if ( (event->button == 1) || (event->button == 3) )
  {
      save_snapshot();
  }
  return TRUE;
}








/**
 * Initialize the main window.
 */
int main( int   argc, char *argv[] )
{
    GtkWidget *vbox;
    GtkWidget *menubar;
    GtkWidget *button;

    grab_init("/dev/video0");

    strcpy( g_tmp_file, ".gtkcamshot.jpg" );
    strcpy( g_output_dir, "/tmp/" );

    strcpy( g_timestamp_font, DEFAULT_FONT  "/16" );

    strcpy( g_format_string, "%T - %e %B %Y" );


    /*
    imlib_context_set_direction(IMLIB_TEXT_TO_RIGHT);
    imlib_context_set_operation(IMLIB_OP_COPY);
    imlib_set_cache_size(0);
    */

    /* Don't allow root to run this program - Gtk programs aren't safe.. */
    if ( 0 == geteuid() )
    {
	fprintf( stderr, "Root shouldn't play this game.\n" );
	exit( -1 );
    }
 
    /* Initialize */
    gtk_init (&argc, &argv);

    /* Create main window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_name (window, "GTKCamShot");


    /* Packing box */
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);
    
    /* Make sure we can exit cleanly */
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);
    

    /* Create and install menu bar */
    menubar = get_menubar_menu (window);
    gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, TRUE, 0);


    /* Button to take an instant snapshot */
    button = gtk_button_new_with_label( "Take Snapshot");
    gtk_box_pack_end( GTK_BOX(vbox), button, TRUE, TRUE, 0 );
    g_signal_connect (G_OBJECT (button), "button_press_event",
		      G_CALLBACK (button_press_event), NULL);

    /* Create the image and display it */
    imageview = gtk_image_new_from_file( g_tmp_file );
    gtk_box_pack_end (GTK_BOX (vbox), imageview, TRUE, TRUE, 0);


    /* Reload the image every three seconds unless otherwise
     * notified.
     */
    gtk_timeout_add(500, get_image, window);

    /* Show the widgets */
    gtk_window_position ((GtkWindow*)window, GTK_WIN_POS_CENTER);
    gtk_widget_show_all (window);


    /* Run the event loop. */
    gtk_main ();


    return 0;
}


