/* -*-mode: C; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  menu.c - Menu bar creation, and menu bar action code.
 *
 *  GTKCamShot - A simple webcam grabbing application
 *
 * Homepage:
 *   http://www.steve.org.uk/Software/gtkcam
 *
 * Author:
 *  Steve Kemp <steve@steve.org.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#include "gtkcamshot.h"




/**
 *
 * Our menu.
 *
 * An array of GtkItemFactoryEntry structures that defines each menu item
 *
 */
static GtkItemFactoryEntry menu_items[] = 
  {
    { "/_File",         NULL,         NULL,           0, "<Branch>" },
    { "/File/_Setup",    "<CTRL>S", file_setup,       0, "<Item>" },
    { "/File/sep1",     NULL,         NULL,           0, "<Separator>" },
    { "/File/_Quit",    "<CTRL>Q", gtk_main_quit,     0, "<StockItem>", GTK_STOCK_QUIT },
    { "/_View",         NULL,         NULL,           0, "<Branch>" },
    { "/View/Timestamp", "<CTRL>T",     view_timestamp_toggle,   1, "<CheckItem>" },
    { "/_Help",         NULL,         NULL,           0, "<LastBranch>" },
    { "/_Help/About",   NULL,         help_about,     0, "<Item>" },
  };


/*
 * Menu item count.
 */
static gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);






/**
 * Display the about dialog box.
 */
void help_about( GtkWidget *w, gpointer   data )
{
    char *authors[] = { "Steve Kemp", NULL };

    GtkAboutDialog *about = gtk_about_dialog_new();

    gtk_about_dialog_set_name(about, "GtkCamShot" );
    gtk_about_dialog_set_authors(about, authors );
    gtk_about_dialog_set_comments(about, "A simple webcam grabbing application");

    gtk_about_dialog_set_website( about, "http://www.steve.org.uk/Software/gtkcamshot" );
    gtk_about_dialog_set_website_label( about, "Homepage" );

    gtk_widget_show( about );
}




/**
 * Display the file setup box. 
 */
void file_setup( GtkWidget *w, gpointer   data )
{
    show_preferences_window();
}



/**
 * Handle the changing of the checkbutton state.
 */
void view_timestamp_toggle( gpointer   callback_data,
                          guint      callback_action,
                          GtkWidget *menu_item )
{
    g_view_timestamp = (GTK_CHECK_MENU_ITEM (menu_item)->active);
}





/**
 * Returns a menubar widget made from the menu above.
 */
GtkWidget *get_menubar_menu( GtkWidget  *window )
{
    GtkItemFactory *item_factory;
    GtkAccelGroup *accel_group;

    /* Make an accelerator group (shortcut keys) */
    accel_group = gtk_accel_group_new ();

    /* Make an ItemFactory (that makes a menubar) */
    item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
					 accel_group);
    
    /* This function generates the menu items. Pass the item factory,
       the number of items in the array, the array itself, and any
       callback data for the the menu items. */
    gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

    /* Attach the new accelerator group to the window. */
    gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
    
    /* Finally, return the actual menu bar created by the item factory. */
    return gtk_item_factory_get_widget (item_factory, "<main>");
}
